# coding: utf-8

"""
Kaziro API

Kaziro Trading API Documentation

The version of the OpenAPI document: 1.0.5
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set, Union

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr, field_validator
from typing_extensions import Self

from ..models.market import Market


class OrderWithMarket(BaseModel):
    """
    OrderWithMarket
    """  # noqa: E501

    id: StrictStr = Field(description="Unique identifier for the created order.")
    date_created: StrictStr = Field(description="Timestamp of order creation.")
    market_id: StrictStr = Field(description="Market identifier for the order.")
    wallet_id: Optional[StrictStr] = Field(default=None, description="Wallet identifier associated with the order.")
    status: StrictStr = Field(description="Current status of the order.")
    probability: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Probability set for the order.")
    size: Union[StrictFloat, StrictInt] = Field(description="Size of the order.")
    order_type: StrictStr = Field(description="Type of the order.")
    expiry_type: StrictStr = Field(description="Expiration type of the order.")
    request_id: Optional[StrictStr] = Field(default=None, description="Request identifier associated with the order.")
    metadata: Optional[Dict[str, Any]] = Field(default=None, description="Metadata associated with the order.")
    market: Optional[Market] = Field(default=None, description="Market associated with the order.")
    __properties: ClassVar[List[str]] = ["id", "date_created", "market_id", "wallet_id", "status", "probability", "size", "order_type", "expiry_type", "request_id", "metadata", "market"]

    @field_validator("status")
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(["OPEN", "CLOSED", "CANCELLED", "MATCHED"]):
            raise ValueError("must be one of enum values ('OPEN', 'CLOSED', 'CANCELLED', 'MATCHED')")
        return value

    @field_validator("order_type")
    def order_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(["MARKET_REQUEST", "MARKET_REPLY"]):
            raise ValueError("must be one of enum values ('MARKET_REQUEST', 'MARKET_REPLY')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of OrderWithMarket from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of market
        if self.market:
            _dict["market"] = self.market.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of OrderWithMarket from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "date_created": obj.get("date_created"),
                "market_id": obj.get("market_id"),
                "wallet_id": obj.get("wallet_id"),
                "status": obj.get("status"),
                "probability": obj.get("probability"),
                "size": obj.get("size"),
                "order_type": obj.get("order_type"),
                "expiry_type": obj.get("expiry_type"),
                "request_id": obj.get("request_id"),
                "metadata": obj.get("metadata"),
                "market": Market.from_dict(obj["market"]) if obj.get("market") is not None else None,
            }
        )
        return _obj



# kaziro/custom/websocket.py
import json
import ssl
import threading
from typing import Callable, Optional

from websocket import WebSocketApp

from ..util.logging import log


class KaziroWebSocket:
    def __init__(self, ws_url: str, api_key: str, verbose: bool = False, verify_ssl: bool = True):
        self.ws_url = ws_url
        self.api_key = api_key
        self.ws: Optional[WebSocketApp] = None
        self.verify_ssl = verify_ssl
        self.connected = False
        self.verbose = verbose
        self.message_callback: Optional[Callable[[str], None]] = None

    def connect(self):
        log("Connecting to WebSocket", verbose=self.verbose)

        if not self.verify_ssl:
            ssl_context = ssl.create_default_context()
            ssl_context.check_hostname = False
            ssl_context.verify_mode = ssl.CERT_NONE

        self.ws = WebSocketApp(
            self.ws_url,
            on_open=self._on_open,
            on_message=self._on_message,
            on_error=self._on_error,
            on_close=self._on_close,
            header={"Kaziro-API-Key": self.api_key},
        )

        if not self.verify_ssl:
            wst = threading.Thread(target=self.ws.run_forever, kwargs={"sslopt": {"cert_reqs": ssl.CERT_NONE}})
        else:
            wst = threading.Thread(target=self.ws.run_forever)
        wst.daemon = True
        wst.start()

    def _on_open(self, ws):
        log("WebSocket connection opened", verbose=self.verbose)
        self.connected = True
        self.subscribe("public:all")

    def _on_message(self, ws, message):
        log(f"Received message: {message}", verbose=self.verbose)
        if self.message_callback:
            self.message_callback(message)

    def _on_error(self, ws, error):
        log(f"WebSocket error: {error}", verbose=self.verbose)

    def _on_close(self, ws, close_status_code, close_msg):
        log("WebSocket connection closed", verbose=self.verbose)
        self.connected = False

    def subscribe(self, channel):
        if self.connected and self.ws:
            self.ws.send(json.dumps({"type": "subscribe", "channel": channel}))
        else:
            log("WebSocket is not connected. Please connect first.", verbose=self.verbose)

    def send(self, message):
        if self.connected and self.ws:
            self.ws.send(json.dumps(message))
        else:
            log("WebSocket is not connected. Please connect first.", verbose=self.verbose)

    def set_message_callback(self, callback: Callable[[str], None]):
        self.message_callback = callback



# coding: utf-8

# flake8: noqa

"""
Kaziro API

Kaziro Trading API Documentation

The version of the OpenAPI document: 1.0.5
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

__version__ = "1.0.0"

# import apis into sdk package
from .api.chat_api import ChatApi
from .api.market_api import MarketApi
from .api.order_api import OrderApi
from .api.position_api import PositionApi
from .api.user_api import UserApi
from .api.wallet_api import WalletApi

# import ApiClient
from .api_response import ApiResponse
from .api_client import ApiClient
from .configuration import Configuration
from .exceptions import OpenApiException
from .exceptions import ApiTypeError
from .exceptions import ApiValueError
from .exceptions import ApiKeyError
from .exceptions import ApiAttributeError
from .exceptions import ApiException

# import models into sdk package
from .models.accept_orders_request import AcceptOrdersRequest
from .models.accept_orders_response import AcceptOrdersResponse
from .models.accepted_order import AcceptedOrder
from .models.chat import Chat
from .models.chat_creation_request import ChatCreationRequest
from .models.chat_retrieval_response import ChatRetrievalResponse
from .models.chat_with_market_and_order import ChatWithMarketAndOrder
from .models.default_reply_order import DefaultReplyOrder
from .models.error_response import ErrorResponse
from .models.error_response_detail import ErrorResponseDetail
from .models.http_validation_error import HTTPValidationError
from .models.market import Market
from .models.market_creation_response import MarketCreationResponse
from .models.market_detail import MarketDetail
from .models.market_ohlc_data import MarketOHLCData
from .models.market_price_history_response import MarketPriceHistoryResponse
from .models.market_retrieval_response import MarketRetrievalResponse
from .models.ohlc_data import OHLCData
from .models.open_orders_response import OpenOrdersResponse
from .models.order import Order
from .models.order_pair import OrderPair
from .models.order_request import OrderRequest
from .models.order_with_market import OrderWithMarket
from .models.place_order_response import PlaceOrderResponse
from .models.position import Position
from .models.position_response import PositionResponse
from .models.position_with_market_and_orders import PositionWithMarketAndOrders
from .models.request_default_replies_request import RequestDefaultRepliesRequest
from .models.request_default_replies_response import RequestDefaultRepliesResponse
from .models.user_profile import UserProfile
from .models.user_profile_response import UserProfileResponse
from .models.validation_error import ValidationError
from .models.validation_error_loc_inner import ValidationErrorLocInner
from .models.wallet import Wallet
from .models.wallet_info_response import WalletInfoResponse



# coding: utf-8

"""
Kaziro API

Kaziro Trading API Documentation

The version of the OpenAPI document: 1.0.5
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing_extensions import Self


class UserProfile(BaseModel):
    """
    UserProfile
    """  # noqa: E501

    id: StrictStr = Field(description="Unique identifier for the user")
    date_created: StrictStr = Field(description="Timestamp of user account creation")
    username: StrictStr = Field(description="User's chosen username")
    display_name: Optional[StrictStr] = Field(default=None, description="User's display name")
    profile_picture: Optional[StrictStr] = Field(default=None, description="URL to user's profile picture")
    metadata: Optional[Dict[str, Any]] = Field(default=None, description="Additional metadata for the user")
    user_type: Optional[StrictStr] = Field(default=None, description="Type of user account")
    bio: Optional[StrictStr] = Field(default=None, description="User's biography")
    __properties: ClassVar[List[str]] = ["id", "date_created", "username", "display_name", "profile_picture", "metadata", "user_type", "bio"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UserProfile from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UserProfile from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "date_created": obj.get("date_created"),
                "username": obj.get("username"),
                "display_name": obj.get("display_name"),
                "profile_picture": obj.get("profile_picture"),
                "metadata": obj.get("metadata"),
                "user_type": obj.get("user_type"),
                "bio": obj.get("bio"),
            }
        )
        return _obj



# coding: utf-8

"""
Kaziro API

Kaziro Trading API Documentation

The version of the OpenAPI document: 1.0.5
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing_extensions import Self


class Market(BaseModel):
    """
    Market
    """  # noqa: E501

    id: StrictStr = Field(description="Unique identifier for the market")
    date_created: StrictStr = Field(description="The date the market was created")
    question: StrictStr = Field(description="The question of the market")
    outcome_1: StrictStr = Field(description="The first outcome of the market")
    outcome_2: StrictStr = Field(description="The second outcome of the market")
    description: StrictStr = Field(description="The description of the market")
    end_date: Optional[StrictStr] = Field(default=None, description="The end date of the market")
    status: StrictStr = Field(description="The status of the market")
    tags: Optional[List[StrictStr]] = Field(default=None, description="The tags of the market")
    creator_id: Optional[StrictStr] = Field(default=None, description="The creator of the market")
    date_closed: Optional[StrictStr] = Field(default=None, description="The date the market was closed")
    result: Optional[StrictStr] = Field(default=None, description="The result of the market. One of None, outcome_1, outcome_2, or void.")
    image_url: Optional[StrictStr] = Field(default=None, description="The URL of the image generated for the market")
    ref_id: Optional[StrictStr] = Field(default=None, description="The reference identifier of the market")
    metadata: Optional[Dict[str, Any]] = Field(default=None, description="Additional metadata for the market. Generally includes order_stat and position_stat")
    __properties: ClassVar[List[str]] = [
        "id",
        "date_created",
        "question",
        "outcome_1",
        "outcome_2",
        "description",
        "end_date",
        "status",
        "tags",
        "creator_id",
        "date_closed",
        "result",
        "image_url",
        "ref_id",
        "metadata",
    ]

    @field_validator("status")
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(["OPEN", "CLOSED", "ACTIVE", "CANCELLED"]):
            raise ValueError("must be one of enum values ('OPEN', 'CLOSED', 'ACTIVE', 'CANCELLED')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Market from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Market from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "date_created": obj.get("date_created"),
                "question": obj.get("question"),
                "outcome_1": obj.get("outcome_1"),
                "outcome_2": obj.get("outcome_2"),
                "description": obj.get("description"),
                "end_date": obj.get("end_date"),
                "status": obj.get("status"),
                "tags": obj.get("tags"),
                "creator_id": obj.get("creator_id"),
                "date_closed": obj.get("date_closed"),
                "result": obj.get("result"),
                "image_url": obj.get("image_url"),
                "ref_id": obj.get("ref_id"),
                "metadata": obj.get("metadata"),
            }
        )
        return _obj



# coding: utf-8

"""
Kaziro API

Kaziro Trading API Documentation

The version of the OpenAPI document: 1.0.5
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from typing import Any, Dict, List, Optional, Tuple, Union

from pydantic import Field, StrictBool, StrictFloat, StrictInt, StrictStr, validate_call
from typing_extensions import Annotated

from ..api_client import ApiClient, RequestSerialized
from ..api_response import ApiResponse
from ..models.accept_orders_request import AcceptOrdersRequest
from ..models.accept_orders_response import AcceptOrdersResponse
from ..models.open_orders_response import OpenOrdersResponse
from ..models.order_request import OrderRequest
from ..models.place_order_response import PlaceOrderResponse
from ..models.request_default_replies_request import RequestDefaultRepliesRequest
from ..models.request_default_replies_response import RequestDefaultRepliesResponse
from ..rest import RESTResponseType


class OrderApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    def accept_orders_endpoint_v1_exchange_order_accept_post(
        self,
        accept_orders_request: AcceptOrdersRequest,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> AcceptOrdersResponse:
        """Accept orders

        Accept multiple open orders for the authenticated user.

        :param accept_orders_request: (required)
        :type accept_orders_request: AcceptOrdersRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._accept_orders_endpoint_v1_exchange_order_accept_post_serialize(
            accept_orders_request=accept_orders_request, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "AcceptOrdersResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "406": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def accept_orders_endpoint_v1_exchange_order_accept_post_with_http_info(
        self,
        accept_orders_request: AcceptOrdersRequest,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[AcceptOrdersResponse]:
        """Accept orders

        Accept multiple open orders for the authenticated user.

        :param accept_orders_request: (required)
        :type accept_orders_request: AcceptOrdersRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._accept_orders_endpoint_v1_exchange_order_accept_post_serialize(
            accept_orders_request=accept_orders_request, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "AcceptOrdersResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "406": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def accept_orders_endpoint_v1_exchange_order_accept_post_without_preload_content(
        self,
        accept_orders_request: AcceptOrdersRequest,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Accept orders

        Accept multiple open orders for the authenticated user.

        :param accept_orders_request: (required)
        :type accept_orders_request: AcceptOrdersRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._accept_orders_endpoint_v1_exchange_order_accept_post_serialize(
            accept_orders_request=accept_orders_request, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "AcceptOrdersResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "406": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        return response_data.response

    def _accept_orders_endpoint_v1_exchange_order_accept_post_serialize(
        self,
        accept_orders_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if accept_orders_request is not None:
            _body_params = accept_orders_request

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params["Content-Type"] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(["application/json"])
            if _default_content_type is not None:
                _header_params["Content-Type"] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader"]

        return self.api_client.param_serialize(
            method="POST",
            resource_path="/v1/exchange/order/accept",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def create_market_endpoint_v1_exchange_order_create_post(
        self,
        order_request: List[OrderRequest],
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PlaceOrderResponse:
        """Place multiple orders

        Create multiple market reply orders for the authenticated user.

        :param order_request: (required)
        :type order_request: List[OrderRequest]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_market_endpoint_v1_exchange_order_create_post_serialize(
            order_request=order_request, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PlaceOrderResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def create_market_endpoint_v1_exchange_order_create_post_with_http_info(
        self,
        order_request: List[OrderRequest],
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PlaceOrderResponse]:
        """Place multiple orders

        Create multiple market reply orders for the authenticated user.

        :param order_request: (required)
        :type order_request: List[OrderRequest]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_market_endpoint_v1_exchange_order_create_post_serialize(
            order_request=order_request, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PlaceOrderResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def create_market_endpoint_v1_exchange_order_create_post_without_preload_content(
        self,
        order_request: List[OrderRequest],
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Place multiple orders

        Create multiple market reply orders for the authenticated user.

        :param order_request: (required)
        :type order_request: List[OrderRequest]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_market_endpoint_v1_exchange_order_create_post_serialize(
            order_request=order_request, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PlaceOrderResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        return response_data.response

    def _create_market_endpoint_v1_exchange_order_create_post_serialize(
        self,
        order_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {
            "OrderRequest": "",
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if order_request is not None:
            _body_params = order_request

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params["Content-Type"] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(["application/json"])
            if _default_content_type is not None:
                _header_params["Content-Type"] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader"]

        return self.api_client.param_serialize(
            method="POST",
            resource_path="/v1/exchange/order/create",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def get_open_orders_endpoint_v1_exchange_order_retrieve_get(
        self,
        order_ids: Optional[List[StrictStr]] = None,
        market_ids: Optional[List[StrictStr]] = None,
        outcome: Optional[StrictInt] = None,
        filter_user: Optional[StrictBool] = None,
        include_markets: Optional[StrictBool] = None,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> OpenOrdersResponse:
        """Retrieve open orders

        Get open orders, optionally filtered by market IDs and user.

        :param order_ids:
        :type order_ids: List[str]
        :param market_ids:
        :type market_ids: List[str]
        :param outcome:
        :type outcome: int
        :param filter_user:
        :type filter_user: bool
        :param include_markets:
        :type include_markets: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_open_orders_endpoint_v1_exchange_order_retrieve_get_serialize(
            order_ids=order_ids,
            market_ids=market_ids,
            outcome=outcome,
            filter_user=filter_user,
            include_markets=include_markets,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "OpenOrdersResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def get_open_orders_endpoint_v1_exchange_order_retrieve_get_with_http_info(
        self,
        order_ids: Optional[List[StrictStr]] = None,
        market_ids: Optional[List[StrictStr]] = None,
        outcome: Optional[StrictInt] = None,
        filter_user: Optional[StrictBool] = None,
        include_markets: Optional[StrictBool] = None,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[OpenOrdersResponse]:
        """Retrieve open orders

        Get open orders, optionally filtered by market IDs and user.

        :param order_ids:
        :type order_ids: List[str]
        :param market_ids:
        :type market_ids: List[str]
        :param outcome:
        :type outcome: int
        :param filter_user:
        :type filter_user: bool
        :param include_markets:
        :type include_markets: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_open_orders_endpoint_v1_exchange_order_retrieve_get_serialize(
            order_ids=order_ids,
            market_ids=market_ids,
            outcome=outcome,
            filter_user=filter_user,
            include_markets=include_markets,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "OpenOrdersResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def get_open_orders_endpoint_v1_exchange_order_retrieve_get_without_preload_content(
        self,
        order_ids: Optional[List[StrictStr]] = None,
        market_ids: Optional[List[StrictStr]] = None,
        outcome: Optional[StrictInt] = None,
        filter_user: Optional[StrictBool] = None,
        include_markets: Optional[StrictBool] = None,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve open orders

        Get open orders, optionally filtered by market IDs and user.

        :param order_ids:
        :type order_ids: List[str]
        :param market_ids:
        :type market_ids: List[str]
        :param outcome:
        :type outcome: int
        :param filter_user:
        :type filter_user: bool
        :param include_markets:
        :type include_markets: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_open_orders_endpoint_v1_exchange_order_retrieve_get_serialize(
            order_ids=order_ids,
            market_ids=market_ids,
            outcome=outcome,
            filter_user=filter_user,
            include_markets=include_markets,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "OpenOrdersResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        return response_data.response

    def _get_open_orders_endpoint_v1_exchange_order_retrieve_get_serialize(
        self,
        order_ids,
        market_ids,
        outcome,
        filter_user,
        include_markets,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {
            "order_ids": "multi",
            "market_ids": "multi",
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if order_ids is not None:
            _query_params.append(("order_ids", order_ids))

        if market_ids is not None:
            _query_params.append(("market_ids", market_ids))

        if outcome is not None:
            _query_params.append(("outcome", outcome))

        if filter_user is not None:
            _query_params.append(("filter_user", filter_user))

        if include_markets is not None:
            _query_params.append(("include_markets", include_markets))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/v1/exchange/order/retrieve",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def request_default_replies_endpoint_v1_exchange_order_temporary_reply_post(
        self,
        request_default_replies_request: RequestDefaultRepliesRequest,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RequestDefaultRepliesResponse:
        """Request default replies

        Request default replies for multiple open requests.

        :param request_default_replies_request: (required)
        :type request_default_replies_request: RequestDefaultRepliesRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._request_default_replies_endpoint_v1_exchange_order_temporary_reply_post_serialize(
            request_default_replies_request=request_default_replies_request, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "RequestDefaultRepliesResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def request_default_replies_endpoint_v1_exchange_order_temporary_reply_post_with_http_info(
        self,
        request_default_replies_request: RequestDefaultRepliesRequest,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[RequestDefaultRepliesResponse]:
        """Request default replies

        Request default replies for multiple open requests.

        :param request_default_replies_request: (required)
        :type request_default_replies_request: RequestDefaultRepliesRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._request_default_replies_endpoint_v1_exchange_order_temporary_reply_post_serialize(
            request_default_replies_request=request_default_replies_request, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "RequestDefaultRepliesResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def request_default_replies_endpoint_v1_exchange_order_temporary_reply_post_without_preload_content(
        self,
        request_default_replies_request: RequestDefaultRepliesRequest,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Request default replies

        Request default replies for multiple open requests.

        :param request_default_replies_request: (required)
        :type request_default_replies_request: RequestDefaultRepliesRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._request_default_replies_endpoint_v1_exchange_order_temporary_reply_post_serialize(
            request_default_replies_request=request_default_replies_request, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "RequestDefaultRepliesResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        return response_data.response

    def _request_default_replies_endpoint_v1_exchange_order_temporary_reply_post_serialize(
        self,
        request_default_replies_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if request_default_replies_request is not None:
            _body_params = request_default_replies_request

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params["Content-Type"] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(["application/json"])
            if _default_content_type is not None:
                _header_params["Content-Type"] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader"]

        return self.api_client.param_serialize(
            method="POST",
            resource_path="/v1/exchange/order/temporary/reply",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



# coding: utf-8

"""
Kaziro API

Kaziro Trading API Documentation

The version of the OpenAPI document: 1.0.5
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from typing import Any, Dict, List, Optional, Tuple, Union

from pydantic import Field, StrictFloat, StrictInt, StrictStr, validate_call
from typing_extensions import Annotated

from ..api_client import ApiClient, RequestSerialized
from ..api_response import ApiResponse
from ..models.market_creation_response import MarketCreationResponse
from ..models.market_detail import MarketDetail
from ..models.market_price_history_response import MarketPriceHistoryResponse
from ..models.market_retrieval_response import MarketRetrievalResponse
from ..rest import RESTResponseType


class MarketApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    def create_market_endpoint_v1_exchange_market_create_post(
        self,
        market_detail: List[MarketDetail],
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> MarketCreationResponse:
        """Create Market Endpoint

        Create multiple markets via API.  This endpoint allows users to create multiple conditional markets simultaneously.  Parameters: - market_details (List[MarketDetail]): A list of market details, each containing:     - detail (str): The description of the market     - ref_id (str): A reference identifier for the market  Returns: - MarketCreationResponse:     A dictionary containing the created markets or an error message.  Raises: - HTTPException(400): If there's a validation error or insufficient balance - HTTPException(401): If the API key is invalid - HTTPException(500): For any other unexpected errors  Note: - This endpoint requires a valid Kaziro API Key to be provided in the header. - A market creation fee will be charged for each market created. - Markets are automatically CLOSED after 10 minutes if no request accepts a reply.

        :param market_detail: (required)
        :type market_detail: List[MarketDetail]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_market_endpoint_v1_exchange_market_create_post_serialize(
            market_detail=market_detail, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "MarketCreationResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def create_market_endpoint_v1_exchange_market_create_post_with_http_info(
        self,
        market_detail: List[MarketDetail],
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[MarketCreationResponse]:
        """Create Market Endpoint

        Create multiple markets via API.  This endpoint allows users to create multiple conditional markets simultaneously.  Parameters: - market_details (List[MarketDetail]): A list of market details, each containing:     - detail (str): The description of the market     - ref_id (str): A reference identifier for the market  Returns: - MarketCreationResponse:     A dictionary containing the created markets or an error message.  Raises: - HTTPException(400): If there's a validation error or insufficient balance - HTTPException(401): If the API key is invalid - HTTPException(500): For any other unexpected errors  Note: - This endpoint requires a valid Kaziro API Key to be provided in the header. - A market creation fee will be charged for each market created. - Markets are automatically CLOSED after 10 minutes if no request accepts a reply.

        :param market_detail: (required)
        :type market_detail: List[MarketDetail]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_market_endpoint_v1_exchange_market_create_post_serialize(
            market_detail=market_detail, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "MarketCreationResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def create_market_endpoint_v1_exchange_market_create_post_without_preload_content(
        self,
        market_detail: List[MarketDetail],
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create Market Endpoint

        Create multiple markets via API.  This endpoint allows users to create multiple conditional markets simultaneously.  Parameters: - market_details (List[MarketDetail]): A list of market details, each containing:     - detail (str): The description of the market     - ref_id (str): A reference identifier for the market  Returns: - MarketCreationResponse:     A dictionary containing the created markets or an error message.  Raises: - HTTPException(400): If there's a validation error or insufficient balance - HTTPException(401): If the API key is invalid - HTTPException(500): For any other unexpected errors  Note: - This endpoint requires a valid Kaziro API Key to be provided in the header. - A market creation fee will be charged for each market created. - Markets are automatically CLOSED after 10 minutes if no request accepts a reply.

        :param market_detail: (required)
        :type market_detail: List[MarketDetail]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_market_endpoint_v1_exchange_market_create_post_serialize(
            market_detail=market_detail, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "MarketCreationResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        return response_data.response

    def _create_market_endpoint_v1_exchange_market_create_post_serialize(
        self,
        market_detail,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {
            "MarketDetail": "",
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if market_detail is not None:
            _body_params = market_detail

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params["Content-Type"] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(["application/json"])
            if _default_content_type is not None:
                _header_params["Content-Type"] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader"]

        return self.api_client.param_serialize(
            method="POST",
            resource_path="/v1/exchange/market/create",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def retrieve_markets_endpoint_v1_exchange_market_retrieve_get(
        self,
        market_ids: Optional[List[StrictStr]] = None,
        statuses: Optional[List[StrictStr]] = None,
        tags: Optional[List[StrictStr]] = None,
        creator_id: Optional[StrictStr] = None,
        search_query: Optional[StrictStr] = None,
        sort_by: Optional[StrictStr] = None,
        sort_direction: Optional[StrictStr] = None,
        bucket: Optional[StrictStr] = None,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> MarketRetrievalResponse:
        """Retrieve Markets Endpoint

        Retrieve markets based on various filters and sorting options.  Parameters: - market_ids: Optional list of specific market IDs to retrieve - status: Filter markets by their status - tags: Filter markets that contain ALL of the specified tags - creator_id: Filter markets by their creator - search_query: Search in market questions and descriptions - sort_by: Field to sort results by - sort_direction: Direction of sort (asc or desc) - bucket: Special grouping parameter for market sets - page: Page number for pagination (starts at 1)  Returns: - MarketRetrievalResponse: A list of markets matching the criteria

        :param market_ids:
        :type market_ids: List[str]
        :param statuses:
        :type statuses: List[str]
        :param tags:
        :type tags: List[str]
        :param creator_id:
        :type creator_id: str
        :param search_query:
        :type search_query: str
        :param sort_by:
        :type sort_by: str
        :param sort_direction:
        :type sort_direction: str
        :param bucket:
        :type bucket: str
        :param page:
        :type page: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._retrieve_markets_endpoint_v1_exchange_market_retrieve_get_serialize(
            market_ids=market_ids,
            statuses=statuses,
            tags=tags,
            creator_id=creator_id,
            search_query=search_query,
            sort_by=sort_by,
            sort_direction=sort_direction,
            bucket=bucket,
            page=page,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "MarketRetrievalResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def retrieve_markets_endpoint_v1_exchange_market_retrieve_get_with_http_info(
        self,
        market_ids: Optional[List[StrictStr]] = None,
        statuses: Optional[List[StrictStr]] = None,
        tags: Optional[List[StrictStr]] = None,
        creator_id: Optional[StrictStr] = None,
        search_query: Optional[StrictStr] = None,
        sort_by: Optional[StrictStr] = None,
        sort_direction: Optional[StrictStr] = None,
        bucket: Optional[StrictStr] = None,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[MarketRetrievalResponse]:
        """Retrieve Markets Endpoint

        Retrieve markets based on various filters and sorting options.  Parameters: - market_ids: Optional list of specific market IDs to retrieve - status: Filter markets by their status - tags: Filter markets that contain ALL of the specified tags - creator_id: Filter markets by their creator - search_query: Search in market questions and descriptions - sort_by: Field to sort results by - sort_direction: Direction of sort (asc or desc) - bucket: Special grouping parameter for market sets - page: Page number for pagination (starts at 1)  Returns: - MarketRetrievalResponse: A list of markets matching the criteria

        :param market_ids:
        :type market_ids: List[str]
        :param statuses:
        :type statuses: List[str]
        :param tags:
        :type tags: List[str]
        :param creator_id:
        :type creator_id: str
        :param search_query:
        :type search_query: str
        :param sort_by:
        :type sort_by: str
        :param sort_direction:
        :type sort_direction: str
        :param bucket:
        :type bucket: str
        :param page:
        :type page: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._retrieve_markets_endpoint_v1_exchange_market_retrieve_get_serialize(
            market_ids=market_ids,
            statuses=statuses,
            tags=tags,
            creator_id=creator_id,
            search_query=search_query,
            sort_by=sort_by,
            sort_direction=sort_direction,
            bucket=bucket,
            page=page,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "MarketRetrievalResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def retrieve_markets_endpoint_v1_exchange_market_retrieve_get_without_preload_content(
        self,
        market_ids: Optional[List[StrictStr]] = None,
        statuses: Optional[List[StrictStr]] = None,
        tags: Optional[List[StrictStr]] = None,
        creator_id: Optional[StrictStr] = None,
        search_query: Optional[StrictStr] = None,
        sort_by: Optional[StrictStr] = None,
        sort_direction: Optional[StrictStr] = None,
        bucket: Optional[StrictStr] = None,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve Markets Endpoint

        Retrieve markets based on various filters and sorting options.  Parameters: - market_ids: Optional list of specific market IDs to retrieve - status: Filter markets by their status - tags: Filter markets that contain ALL of the specified tags - creator_id: Filter markets by their creator - search_query: Search in market questions and descriptions - sort_by: Field to sort results by - sort_direction: Direction of sort (asc or desc) - bucket: Special grouping parameter for market sets - page: Page number for pagination (starts at 1)  Returns: - MarketRetrievalResponse: A list of markets matching the criteria

        :param market_ids:
        :type market_ids: List[str]
        :param statuses:
        :type statuses: List[str]
        :param tags:
        :type tags: List[str]
        :param creator_id:
        :type creator_id: str
        :param search_query:
        :type search_query: str
        :param sort_by:
        :type sort_by: str
        :param sort_direction:
        :type sort_direction: str
        :param bucket:
        :type bucket: str
        :param page:
        :type page: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._retrieve_markets_endpoint_v1_exchange_market_retrieve_get_serialize(
            market_ids=market_ids,
            statuses=statuses,
            tags=tags,
            creator_id=creator_id,
            search_query=search_query,
            sort_by=sort_by,
            sort_direction=sort_direction,
            bucket=bucket,
            page=page,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "MarketRetrievalResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        return response_data.response

    def _retrieve_markets_endpoint_v1_exchange_market_retrieve_get_serialize(
        self,
        market_ids,
        statuses,
        tags,
        creator_id,
        search_query,
        sort_by,
        sort_direction,
        bucket,
        page,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {
            "market_ids": "multi",
            "statuses": "multi",
            "tags": "multi",
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if market_ids is not None:
            _query_params.append(("market_ids", market_ids))

        if statuses is not None:
            _query_params.append(("statuses", statuses))

        if tags is not None:
            _query_params.append(("tags", tags))

        if creator_id is not None:
            _query_params.append(("creator_id", creator_id))

        if search_query is not None:
            _query_params.append(("search_query", search_query))

        if sort_by is not None:
            _query_params.append(("sort_by", sort_by))

        if sort_direction is not None:
            _query_params.append(("sort_direction", sort_direction))

        if bucket is not None:
            _query_params.append(("bucket", bucket))

        if page is not None:
            _query_params.append(("page", page))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/v1/exchange/market/retrieve",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def retrieve_ohlc_endpoint_v1_exchange_market_retrieve_ohlc_get(
        self,
        market_ids: Annotated[List[StrictStr], Field(description="List of market IDs to retrieve OHLC data for")],
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> MarketPriceHistoryResponse:
        """Retrieve Ohlc Endpoint


        :param market_ids: List of market IDs to retrieve OHLC data for (required)
        :type market_ids: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._retrieve_ohlc_endpoint_v1_exchange_market_retrieve_ohlc_get_serialize(
            market_ids=market_ids, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "MarketPriceHistoryResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def retrieve_ohlc_endpoint_v1_exchange_market_retrieve_ohlc_get_with_http_info(
        self,
        market_ids: Annotated[List[StrictStr], Field(description="List of market IDs to retrieve OHLC data for")],
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[MarketPriceHistoryResponse]:
        """Retrieve Ohlc Endpoint


        :param market_ids: List of market IDs to retrieve OHLC data for (required)
        :type market_ids: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._retrieve_ohlc_endpoint_v1_exchange_market_retrieve_ohlc_get_serialize(
            market_ids=market_ids, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "MarketPriceHistoryResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def retrieve_ohlc_endpoint_v1_exchange_market_retrieve_ohlc_get_without_preload_content(
        self,
        market_ids: Annotated[List[StrictStr], Field(description="List of market IDs to retrieve OHLC data for")],
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve Ohlc Endpoint


        :param market_ids: List of market IDs to retrieve OHLC data for (required)
        :type market_ids: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._retrieve_ohlc_endpoint_v1_exchange_market_retrieve_ohlc_get_serialize(
            market_ids=market_ids, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "MarketPriceHistoryResponse",
            "400": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        return response_data.response

    def _retrieve_ohlc_endpoint_v1_exchange_market_retrieve_ohlc_get_serialize(
        self,
        market_ids,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {
            "market_ids": "multi",
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if market_ids is not None:
            _query_params.append(("market_ids", market_ids))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/v1/exchange/market/retrieve/ohlc",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



# kaziro/templates/bots.py
import json
import time
from typing import Literal


class BaseReplierTemplate:
    def __init__(self, kaziro):
        print("Initializing BaseReplierTemplate...")
        self.kaziro = kaziro
        self.wallet = self.kaziro.wallet.retrieve().wallets[0]
        print(f"Wallet Balance: {self.wallet.balance}")

    def start(self):
        print("Starting BaseReplierTemplate...")

        def on_message(message: str):
            data = json.loads(message)
            if data.get("event_type") == "order_created":
                order = data.get("data", {}).get("order", {})
                if order.get("order_type") == "MARKET_REPLY":
                    print("Market reply order detected...")
                    request_id = order.get("request_id")
                    probability = order.get("probability")
                    initializer = order.get("initializer")
                    outcome = order.get("outcome")
                    size = order.get("size")
                    if request_id and probability is not None and initializer and size:
                        new_probability = max(0.02, probability - 0.01)
                        if validate_balance(probability, new_probability, size):
                            if validate_request(request_id):
                                create_order(request_id, new_probability, outcome)
                            else:
                                print("Request validation failed")
                        else:
                            print("Balance validation failed")
                    else:
                        # print("Missing required order details")
                        pass
                else:
                    # print(f"Non-market reply order type: {order.get('order_type')}")
                    pass
            else:
                # print(f"Non-order created event type: {data.get('event_type')}")
                pass

        def validate_balance(existing_probability: float, new_probability: float, size: float, allowed_risk: float = 0.3):
            implied_request_size = round(size / (1 - existing_probability) - size, 2)
            expected_reply_size = round((1 / new_probability) * implied_request_size - implied_request_size, 2)
            # Get realtime balance
            current_balance = self.kaziro.wallet.retrieve().wallets[0].balance
            if (current_balance * allowed_risk) < expected_reply_size:
                print(f"Insufficient balance for allowed risk. Max allowed risk: {current_balance * allowed_risk}")
                return False
            return True

        def validate_request(request_id: str):
            print(f"Validating request: request_id={request_id}")
            response = self.kaziro.order.retrieve(order_ids=[request_id])
            if response.success and response.orders:
                request = response.orders[0]
                if request.wallet_id == self.wallet.id:
                    print("Cannot reply to own request")
                    return False
                return True
            else:
                print(f"Failed to retrieve request or no orders found: {response}")
                return False

        def create_order(request_id: str, probability: float, outcome: Literal[1, 2]):
            try:
                response = self.kaziro.order.create_single_reply(request_id=request_id, probability=probability, outcome=outcome)
                print(f"Order created successfully: {response}")
            except Exception as e:
                print(f"Error creating order: {str(e)}")
                print(f"Error type: {type(e).__name__}")

        self.kaziro.set_websocket_callback(on_message)
        self.kaziro.connect_websocket()
        self.kaziro.subscribe_websocket("public:all")
        print("Market Maker Bot started. Press Ctrl+C to stop.")
        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            print("Bot stopped.")



# coding: utf-8

"""
Kaziro API

Kaziro Trading API Documentation

The version of the OpenAPI document: 1.0.5
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from typing import Any, Dict, List, Optional, Tuple, Union

from pydantic import Field, StrictFloat, StrictInt, StrictStr, validate_call
from typing_extensions import Annotated

from ..api_client import ApiClient, RequestSerialized
from ..api_response import ApiResponse
from ..models.user_profile_response import UserProfileResponse
from ..rest import RESTResponseType


class UserApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    def profile_retrieve_endpoint_v1_user_retrieve_get(
        self,
        user_ids: Annotated[Optional[List[StrictStr]], Field(description="List of user IDs to retrieve profiles for")] = None,
        usernames: Annotated[Optional[List[StrictStr]], Field(description="List of usernames to retrieve profiles for")] = None,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UserProfileResponse:
        """Profile Retrieve Endpoint

        Retrieve user profiles by either user IDs or usernames.  Parameters: - user_ids: Optional list of user IDs - usernames: Optional list of usernames  If no user_ids or usernames are provided, the user profile for the authenticated user will be retrieved. If no authenticated user is provided, an error will be returned.  Returns profiles for the requested users. Must provide either user_ids or usernames.

        :param user_ids: List of user IDs to retrieve profiles for
        :type user_ids: List[str]
        :param usernames: List of usernames to retrieve profiles for
        :type usernames: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._profile_retrieve_endpoint_v1_user_retrieve_get_serialize(
            user_ids=user_ids, usernames=usernames, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "UserProfileResponse",
            "400": "ErrorResponse",
            "404": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def profile_retrieve_endpoint_v1_user_retrieve_get_with_http_info(
        self,
        user_ids: Annotated[Optional[List[StrictStr]], Field(description="List of user IDs to retrieve profiles for")] = None,
        usernames: Annotated[Optional[List[StrictStr]], Field(description="List of usernames to retrieve profiles for")] = None,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UserProfileResponse]:
        """Profile Retrieve Endpoint

        Retrieve user profiles by either user IDs or usernames.  Parameters: - user_ids: Optional list of user IDs - usernames: Optional list of usernames  If no user_ids or usernames are provided, the user profile for the authenticated user will be retrieved. If no authenticated user is provided, an error will be returned.  Returns profiles for the requested users. Must provide either user_ids or usernames.

        :param user_ids: List of user IDs to retrieve profiles for
        :type user_ids: List[str]
        :param usernames: List of usernames to retrieve profiles for
        :type usernames: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._profile_retrieve_endpoint_v1_user_retrieve_get_serialize(
            user_ids=user_ids, usernames=usernames, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "UserProfileResponse",
            "400": "ErrorResponse",
            "404": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def profile_retrieve_endpoint_v1_user_retrieve_get_without_preload_content(
        self,
        user_ids: Annotated[Optional[List[StrictStr]], Field(description="List of user IDs to retrieve profiles for")] = None,
        usernames: Annotated[Optional[List[StrictStr]], Field(description="List of usernames to retrieve profiles for")] = None,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Profile Retrieve Endpoint

        Retrieve user profiles by either user IDs or usernames.  Parameters: - user_ids: Optional list of user IDs - usernames: Optional list of usernames  If no user_ids or usernames are provided, the user profile for the authenticated user will be retrieved. If no authenticated user is provided, an error will be returned.  Returns profiles for the requested users. Must provide either user_ids or usernames.

        :param user_ids: List of user IDs to retrieve profiles for
        :type user_ids: List[str]
        :param usernames: List of usernames to retrieve profiles for
        :type usernames: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._profile_retrieve_endpoint_v1_user_retrieve_get_serialize(
            user_ids=user_ids, usernames=usernames, _request_auth=_request_auth, _content_type=_content_type, _headers=_headers, _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "UserProfileResponse",
            "400": "ErrorResponse",
            "404": "ErrorResponse",
            "500": "ErrorResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        return response_data.response

    def _profile_retrieve_endpoint_v1_user_retrieve_get_serialize(
        self,
        user_ids,
        usernames,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {
            "user_ids": "multi",
            "usernames": "multi",
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if user_ids is not None:
            _query_params.append(("user_ids", user_ids))

        if usernames is not None:
            _query_params.append(("usernames", usernames))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/v1/user/retrieve",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



# coding: utf-8

"""
Kaziro API

Kaziro Trading API Documentation

The version of the OpenAPI document: 1.0.5
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set, Union

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing_extensions import Self


class Position(BaseModel):
    """
    Position
    """  # noqa: E501

    id: StrictStr = Field(description="Unique identifier for the position")
    market_id: StrictStr = Field(description="ID of the market this position belongs to")
    status: StrictStr = Field(description="Status of the position")
    side: StrictStr = Field(description="Side of the position based on the caller's user_id")
    request_size: Union[StrictFloat, StrictInt] = Field(description="Size of the request order")
    request_id: Optional[StrictStr] = Field(default=None, description="ID of the request order (order_id)")
    reply_size: Union[StrictFloat, StrictInt] = Field(description="Size of the reply order")
    reply_id: Optional[StrictStr] = Field(default=None, description="ID of the reply order (order_id)")
    protocol_id: StrictInt = Field(description="On-chain ID of the position")
    metadata: Optional[Dict[str, Any]] = Field(default=None, description="Additional metadata for the position")
    outcome: Optional[StrictInt] = Field(default=None, description="Result of the position. 1 for outcome 1, 2 for outcome 2, 0 for not yet determined, 3 for void")
    outcome_probability: Optional[Union[StrictFloat, StrictInt]] = Field(
        default=None, description="Probability of outcome 1. Provided for convenience. Can be determined from the size of the request and reply orders"
    )
    __properties: ClassVar[List[str]] = [
        "id",
        "market_id",
        "status",
        "side",
        "request_size",
        "request_id",
        "reply_size",
        "reply_id",
        "protocol_id",
        "metadata",
        "outcome",
        "outcome_probability",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Position from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Position from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "market_id": obj.get("market_id"),
                "status": obj.get("status"),
                "side": obj.get("side"),
                "request_size": obj.get("request_size"),
                "request_id": obj.get("request_id"),
                "reply_size": obj.get("reply_size"),
                "reply_id": obj.get("reply_id"),
                "protocol_id": obj.get("protocol_id"),
                "metadata": obj.get("metadata"),
                "outcome": obj.get("outcome"),
                "outcome_probability": obj.get("outcome_probability"),
            }
        )
        return _obj



# kaziro/__init__.py

from typing import Callable, List, Literal, Optional

import urllib3

from .api.generated.kaziro import ApiClient, Configuration
from .api.generated.kaziro.api import ChatApi, MarketApi, OrderApi, PositionApi, UserApi, WalletApi
from .api.generated.kaziro.models import (
    AcceptOrdersRequest,
    AcceptOrdersResponse,
    Chat,
    ChatCreationRequest,
    ChatRetrievalResponse,
    MarketCreationResponse,
    MarketDetail,
    MarketPriceHistoryResponse,
    MarketRetrievalResponse,
    OpenOrdersResponse,
    OrderPair,
    OrderRequest,
    PlaceOrderResponse,
    PositionResponse,
    RequestDefaultRepliesRequest,
    RequestDefaultRepliesResponse,
    UserProfileResponse,
    WalletInfoResponse,
)
from .template.base_replier import BaseReplierTemplate  # noqa: F401
from .websocket.client import KaziroWebSocket


class Kaziro:
    def __init__(self, api_key: str, api_url: str = "https://api.kaziro.xyz", ws_url: str = "wss://ws.kaziro.xyz", verbose: bool = False, verify_ssl: bool = True):
        self.api_key = api_key
        self.api_url = api_url
        self.ws_url = ws_url
        self.verbose = verbose
        self.verify_ssl = verify_ssl
        # Initialize Configuration and ApiClient
        self.config = Configuration(host=self.api_url, api_key={"APIKeyHeader": self.api_key})
        self.config.verify_ssl = verify_ssl
        if not verify_ssl:
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        self.api_client = ApiClient(configuration=self.config)

        # Initialize API classes
        self.user = self.User(self.api_client)
        self.market = self.Market(self.api_client)
        self.order = self.Order(self.api_client)
        self.position = self.Position(self.api_client)
        self.wallet = self.Wallet(self.api_client)
        self.chat = self.Chat(self.api_client)

        # Initialize WebSocket
        self.ws = KaziroWebSocket(self.ws_url, self.api_key, verbose=self.verbose, verify_ssl=self.verify_ssl)

    class User:
        def __init__(self, api_client: ApiClient):
            self.api = UserApi(api_client)

        def retrieve(self, user_ids: Optional[List[str]] = None, usernames: Optional[List[str]] = None) -> UserProfileResponse:
            return self.api.profile_retrieve_endpoint_v1_user_retrieve_get(user_ids, usernames)

    class Market:
        def __init__(self, api_client: ApiClient):
            self.api = MarketApi(api_client)

        def create(self, market_details: List[MarketDetail]) -> MarketCreationResponse:
            return self.api.create_market_endpoint_v1_exchange_market_create_post(market_details)

        def create_single(self, detail: str) -> MarketCreationResponse:
            market_detail = MarketDetail(detail=detail)
            response = self.api.create_market_endpoint_v1_exchange_market_create_post([market_detail])
            return response

        def retrieve(
            self,
            market_ids: Optional[List[str]] = None,
            statuses: Optional[List[str]] = None,
            tags: Optional[List[str]] = None,
            creator_id: Optional[str] = None,
            search_query: Optional[str] = None,
            sort_by: Optional[str] = None,
            sort_direction: Optional[str] = None,
            bucket: Optional[str] = None,
            page: Optional[int] = None,
        ) -> MarketRetrievalResponse:
            return self.api.retrieve_markets_endpoint_v1_exchange_market_retrieve_get(market_ids, statuses, tags, creator_id, search_query, sort_by, sort_direction, bucket, page)

        def retrieve_ohlc(self, market_ids: List[str]) -> MarketPriceHistoryResponse:
            return self.api.retrieve_ohlc_endpoint_v1_exchange_market_retrieve_ohlc_get(market_ids)

        def retrieve_ohlc_single(self, market_id: str) -> MarketPriceHistoryResponse:
            return self.api.retrieve_ohlc_endpoint_v1_exchange_market_retrieve_ohlc_get([market_id])

    class Order:
        def __init__(self, api_client: ApiClient):
            self.api = OrderApi(api_client)

        def create(self, orders: List[OrderRequest]) -> PlaceOrderResponse:
            return self.api.create_market_endpoint_v1_exchange_order_create_post(orders)

        def create_single_request(self, size: float, market_id: str, outcome: Literal[1, 2]) -> PlaceOrderResponse:
            order = OrderRequest(size=size, market_id=market_id, order_type="MARKET_REQUEST", outcome=outcome)
            return self.api.create_market_endpoint_v1_exchange_order_create_post([order])

        def create_single_reply(self, request_id: Optional[str], probability: float, outcome: Literal[1, 2]) -> PlaceOrderResponse:
            order = OrderRequest(request_id=request_id, probability=probability, order_type="MARKET_REPLY", outcome=outcome)
            return self.api.create_market_endpoint_v1_exchange_order_create_post([order])

        def retrieve(
            self,
            order_ids: Optional[List[str]] = None,
            market_ids: Optional[List[str]] = None,
            outcome: Optional[Literal[1, 2]] = None,
            filter_user: bool = False,
            include_markets: bool = False,
        ) -> OpenOrdersResponse:
            return self.api.get_open_orders_endpoint_v1_exchange_order_retrieve_get(order_ids, market_ids, outcome, filter_user, include_markets)

        def accept(self, orders: List[OrderPair]) -> AcceptOrdersResponse:
            accept_orders_request = AcceptOrdersRequest(orders=orders)
            return self.api.accept_orders_endpoint_v1_exchange_order_accept_post(accept_orders_request)

        def accept_single(self, request_id: str, reply_id: str) -> AcceptOrdersResponse:
            order_pair = OrderPair(request_id=request_id, reply_id=reply_id)
            return self.accept([order_pair])

        def request_default_replies(self, order_ids: List[str]) -> RequestDefaultRepliesResponse:
            request_default_replies_request = RequestDefaultRepliesRequest(order_ids=order_ids)
            return self.api.request_default_replies_endpoint_v1_exchange_order_temporary_reply_post(request_default_replies_request)

        def request_default_reply_single(self, order_id: str) -> RequestDefaultRepliesResponse:
            return self.request_default_replies([order_id])

    class Position:
        def __init__(self, api_client: ApiClient):
            self.api = PositionApi(api_client)

        def retrieve(
            self, status: Optional[str] = None, page: Optional[int] = None, sort_order: Optional[str] = None, include_markets: bool = False, include_orders: bool = False
        ) -> PositionResponse:
            return self.api.get_positions_v1_exchange_position_retrieve_get(status=status, page=page, sort_order=sort_order, include_markets=include_markets, include_orders=include_orders)

    class Wallet:
        def __init__(self, api_client: ApiClient):
            self.api = WalletApi(api_client)

        def retrieve(self) -> WalletInfoResponse:
            return self.api.wallet_info_endpoint_v1_wallet_retrieve_get()

    class Chat:
        def __init__(self, api_client: ApiClient):
            self.api = ChatApi(api_client)

        def create(self, content: str, market_id: Optional[str] = None) -> Chat:
            """Create a new chat message.

            Args:
                content: The content of the chat message
                market_id: Optional market ID to associate with the message

            Returns:
                Chat: The created chat message
            """
            chat_request = ChatCreationRequest(content=content, market_id=market_id)
            return self.api.create_chat_message_endpoint_v1_exchange_chat_create_post(chat_request)

        def retrieve(
            self,
            market_id: Optional[str] = None,
            order_id: Optional[str] = None,
            user_id: Optional[str] = None,
            is_global: Optional[bool] = None,
            limit: Optional[int] = None,
            before_date: Optional[str] = None,
            include_market: Optional[bool] = None,
        ) -> ChatRetrievalResponse:
            """Retrieve chat messages with optional filters.

            Args:
                market_id: Filter by market ID
                order_id: Filter by order ID
                user_id: Filter by user ID
                is_global: Filter by global status
                limit: Maximum number of messages to return (max 100)
                before_date: Return messages before this date
                include_market: Include full market data in response

            Returns:
                ChatRetrievalResponse: The retrieved chat messages
            """
            return self.api.retrieve_chat_messages_endpoint_v1_exchange_chat_retrieve_get(
                market_id=market_id, order_id=order_id, user_id=user_id, is_global=is_global, limit=limit, before_date=before_date, include_market=include_market
            )

    def connect_websocket(self):
        self.ws.connect()

    def subscribe_websocket(self, channel: str):
        self.ws.subscribe(channel)

    def set_websocket_callback(self, callback: Callable[[str], None]):
        self.ws.set_message_callback(callback)


__all__ = ["Kaziro"]



# coding: utf-8

"""
Kaziro API

Kaziro Trading API Documentation

The version of the OpenAPI document: 1.0.5
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing_extensions import Self

from ..models.market import Market
from ..models.order import Order


class ChatWithMarketAndOrder(BaseModel):
    """
    ChatWithMarketAndOrder
    """  # noqa: E501

    id: StrictStr = Field(description="Unique identifier for the chat message")
    user_id: StrictStr = Field(description="User identifier associated with the chat message")
    content: StrictStr = Field(description="The content of the chat message")
    date_created: StrictStr = Field(description="The date the message was created")
    metadata: Optional[Dict[str, Any]] = Field(default=None, description="Additional metadata for the chat message")
    market_id: Optional[StrictStr] = Field(default=None, description="Market identifier associated with the chat message")
    order_id: Optional[StrictStr] = Field(default=None, description="Order identifier associated with the chat message")
    is_global: StrictBool = Field(
        description="Whether the chat message is global. A global chat message is visible to all users on Kaziro and is not associated with a particular market or order, while a non-global chat message is visible to the user who sent it and the creator of the market or order"
    )
    market: Optional[Market] = None
    order: Optional[Order] = None
    __properties: ClassVar[List[str]] = ["id", "user_id", "content", "date_created", "metadata", "market_id", "order_id", "is_global", "market", "order"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ChatWithMarketAndOrder from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of market
        if self.market:
            _dict["market"] = self.market.to_dict()
        # override the default output from pydantic by calling `to_dict()` of order
        if self.order:
            _dict["order"] = self.order.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ChatWithMarketAndOrder from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "user_id": obj.get("user_id"),
                "content": obj.get("content"),
                "date_created": obj.get("date_created"),
                "metadata": obj.get("metadata"),
                "market_id": obj.get("market_id"),
                "order_id": obj.get("order_id"),
                "is_global": obj.get("is_global"),
                "market": Market.from_dict(obj["market"]) if obj.get("market") is not None else None,
                "order": Order.from_dict(obj["order"]) if obj.get("order") is not None else None,
            }
        )
        return _obj



import logging

# Create a logger for this package without configuring it
logger = logging.getLogger("kaziro")


def log(*args, level=logging.INFO, verbose=False):
    """
    Log function that only logs messages when verbose is True.

    Args:
        *args: Variable length argument list, similar to print function.
        level: The logging level (e.g., logging.INFO, logging.DEBUG). Default is logging.INFO.
        verbose (bool): If True, the message will be logged. If False, nothing will be logged.
    """
    if verbose:
        message = " ".join(map(str, args))
        logger.log(level, message)


def setup_logger(name, debug=False):
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG if debug else logging.INFO)

    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")

    console_handler = logging.StreamHandler()
    console_handler.setFormatter(formatter)

    logger.addHandler(console_handler)

    return logger



# coding: utf-8

# flake8: noqa
"""
Kaziro API

Kaziro Trading API Documentation

The version of the OpenAPI document: 1.0.5
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

# import models into model package
from ..models.accept_orders_request import AcceptOrdersRequest
from ..models.accept_orders_response import AcceptOrdersResponse
from ..models.accepted_order import AcceptedOrder
from ..models.chat import Chat
from ..models.chat_creation_request import ChatCreationRequest
from ..models.chat_retrieval_response import ChatRetrievalResponse
from ..models.chat_with_market_and_order import ChatWithMarketAndOrder
from ..models.default_reply_order import DefaultReplyOrder
from ..models.error_response import ErrorResponse
from ..models.error_response_detail import ErrorResponseDetail
from ..models.http_validation_error import HTTPValidationError
from ..models.market import Market
from ..models.market_creation_response import MarketCreationResponse
from ..models.market_detail import MarketDetail
from ..models.market_ohlc_data import MarketOHLCData
from ..models.market_price_history_response import MarketPriceHistoryResponse
from ..models.market_retrieval_response import MarketRetrievalResponse
from ..models.ohlc_data import OHLCData
from ..models.open_orders_response import OpenOrdersResponse
from ..models.order import Order
from ..models.order_pair import OrderPair
from ..models.order_request import OrderRequest
from ..models.order_with_market import OrderWithMarket
from ..models.place_order_response import PlaceOrderResponse
from ..models.position import Position
from ..models.position_response import PositionResponse
from ..models.position_with_market_and_orders import PositionWithMarketAndOrders
from ..models.request_default_replies_request import RequestDefaultRepliesRequest
from ..models.request_default_replies_response import RequestDefaultRepliesResponse
from ..models.user_profile import UserProfile
from ..models.user_profile_response import UserProfileResponse
from ..models.validation_error import ValidationError
from ..models.validation_error_loc_inner import ValidationErrorLocInner
from ..models.wallet import Wallet
from ..models.wallet_info_response import WalletInfoResponse



# flake8: noqa

# import apis into api package
from ..api.chat_api import ChatApi
from ..api.market_api import MarketApi
from ..api.order_api import OrderApi
from ..api.position_api import PositionApi
from ..api.user_api import UserApi
from ..api.wallet_api import WalletApi
